@@ -13,18 +13,20 @@ import os
 import sys
 import time
 from qt.core import (
-    QAction, QApplication, QDialog, QFont, QImage, QItemSelectionModel,
-    QKeySequence, QLabel, QSize, QSizePolicy, QStackedLayout, Qt, QTimer, pyqtSignal
+    QAction, QApplication, QDialog, QFont, QImage, QItemSelectionModel, QKeySequence,
+    QLabel, QPainter, QPalette, QSize, QSizePolicy, QStackedLayout, Qt, QTimer,
+    pyqtSignal,
 )
 
 from calibre.constants import islinux
-from calibre.ebooks.metadata import rating_to_stars
-from calibre.gui2 import (
-    available_height, available_width, config, gprefs, rating_font
-)
+from calibre.ebooks.metadata import authors_to_string, rating_to_stars
+from calibre.gui2 import config, gprefs, rating_font
 from calibre_extensions import pictureflow
 
 
+MIN_SIZE = QSize(300, 150)
+
+
 class EmptyImageList(pictureflow.FlowImages):
 
     def __init__(self):
@@ -150,6 +152,16 @@ class DatabaseImages(pictureflow.FlowImages):
                     if val:
                         return rating_to_stars(val, allow_half_stars=db.field_metadata[field]['display'].get('allow_half_stars'))
                 else:
+                    if field == 'authors':
+                        book_id = self.model.id(index)
+                        val = db.field_for(field, book_id, default_value=0)
+                        if val == (_('Unknown'),):
+                            val = ''
+                        elif val:
+                            val = authors_to_string(val).replace('&', '&&')
+                        else:
+                            val = ''
+                        return val
                     return self.render_template('{%s}' % field, index, db).replace('&', '&&')
         except Exception:
             if not self.subtitle_error_reported:
@@ -178,11 +190,38 @@ class CoverFlow(pictureflow.PictureFlow):
 
     dc_signal = pyqtSignal()
     context_menu_requested = pyqtSignal()
+    _ignore_paint_events = False
+
+    @property
+    def ignore_paint_events(self):
+        return self._ignore_paint_events
+
+    @ignore_paint_events.setter
+    def ignore_paint_events(self, val):
+        if val != self._ignore_paint_events:
+            self._ignore_paint_events = val
+            if not val:
+                self.update()
+
+    def paintEvent(self, ev):
+        if self.ignore_paint_events and time.monotonic() - self.created_at < 1:
+            # Paint blank during startup to avoid flashing
+            p = QPainter(self)
+            p.fillRect(self.rect(), self.palette().color(QPalette.ColorRole.Window))
+            p.end()
+        else:
+            super().paintEvent(ev)
+
+    def resizeEvent(self, ev):
+        if ev.oldSize() == QSize(-1, -1):
+            self.created_at = time.monotonic()
+        super().resizeEvent(ev)
 
     def __init__(self, parent=None):
         pictureflow.PictureFlow.__init__(self, parent,
                             config['cover_flow_queue_length']+1)
-        self.setMinimumSize(QSize(300, 150))
+        self.created_at = time.monotonic()
+        self.setMinimumSize(MIN_SIZE)
         self.setFocusPolicy(Qt.FocusPolicy.WheelFocus)
         self.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Expanding,
             QSizePolicy.Policy.Expanding))
@@ -223,7 +262,10 @@ class CoverFlow(pictureflow.PictureFlow):
         return self.minimumSize()
 
     def wheelEvent(self, ev):
-        d = ev.angleDelta().y()
+        if abs(ev.angleDelta().x()) > abs(ev.angleDelta().y()):
+            d = ev.angleDelta().x()
+        else:
+            d = ev.angleDelta().y()
         if abs(d) > 0:
             ev.accept()
             (self.showNext if d < 0 else self.showPrevious)()
@@ -249,10 +291,7 @@ class CBDialog(QDialog):
         self.setWindowTitle(_('Browse by covers'))
         self.layout().addWidget(cover_flow)
 
-        geom = gprefs.get('cover_browser_dialog_geometry', None)
-        if not geom or not QApplication.instance().safe_restore_geometry(self, geom):
-            h, w = available_height()-60, int(available_width()/1.5)
-            self.resize(w, h)
+        self.restore_geometry(gprefs, 'cover_browser_dialog_geometry')
         self.action_fs_toggle = a = QAction(self)
         self.addAction(a)
         a.setShortcuts([QKeySequence(QKeySequence.StandardKey.FullScreen)])
@@ -282,10 +321,15 @@ class CBDialog(QDialog):
             menuless_qaction.shortcuts()))
         a.triggered.connect(iactions['Send To Device'].menuless_qaction.trigger)
 
+    def sizeHint(self):
+        sz = self.screen().availableSize()
+        sz.setHeight(sz.height()-60)
+        sz.setWidth(int(sz.width()/1.5))
+        return sz
+
     def closeEvent(self, *args):
         if not self.isFullScreen():
-            geom = bytearray(self.saveGeometry())
-            gprefs['cover_browser_dialog_geometry'] = geom
+            self.save_geometry(gprefs, 'cover_browser_dialog_geometry')
         self.closed.emit()
 
     def show_normal(self):
@@ -309,9 +353,6 @@ class CoverFlowMixin:
 
     disable_cover_browser_refresh = False
 
-    def __init__(self, *args, **kwargs):
-        pass
-
     def one_auto_scroll(self):
         cb_visible = self.cover_flow is not None and self.cb_splitter.button.isChecked()
         if cb_visible:
@@ -334,11 +375,11 @@ class CoverFlowMixin:
             self.auto_scroll_timer.stop()
             self.toggle_auto_scroll()
 
-    def init_cover_flow_mixin(self):
-        self.cover_flow = None
+    def __init__(self, *a, **kw):
         self.cf_last_updated_at = None
         self.cover_flow_syncing_enabled = False
         self.cover_flow_sync_flag = True
+        self.separate_cover_browser = config['separate_cover_flow']
         self.cover_flow = CoverFlow(parent=self)
         self.cover_flow.currentChanged.connect(self.sync_listview_to_cf)
         self.cover_flow.context_menu_requested.connect(self.cf_context_menu_requested)
@@ -347,19 +388,17 @@ class CoverFlowMixin:
         self.cover_flow.setImages(self.db_images)
         self.cover_flow.itemActivated.connect(self.iactions['View'].view_specific_book)
         self.update_cover_flow_subtitle_font()
-        if config['separate_cover_flow']:
+        if self.separate_cover_browser:
             self.separate_cover_browser = True
             self.cb_splitter.button.clicked.connect(self.toggle_cover_browser)
             self.cb_splitter.button.set_state_to_show()
             self.cb_splitter.action_toggle.triggered.connect(self.toggle_cover_browser)
-            if CoverFlow is not None:
-                self.cover_flow.stop.connect(self.hide_cover_browser)
+            self.cover_flow.stop.connect(self.hide_cover_browser)
             self.cover_flow.setVisible(False)
         else:
             self.separate_cover_browser = False
             self.cb_splitter.insertWidget(self.cb_splitter.side_index, self.cover_flow)
-            if CoverFlow is not None:
-                self.cover_flow.stop.connect(self.cb_splitter.hide_side_pane)
+            self.cover_flow.stop.connect(self.cb_splitter.hide_side_pane)
         self.cb_splitter.button.toggled.connect(self.cover_browser_toggled, type=Qt.ConnectionType.QueuedConnection)
 
     def update_cover_flow_subtitle_font(self):
@@ -387,26 +426,24 @@ class CoverFlowMixin:
 
     def cover_browser_shown(self):
         self.cover_flow.setFocus(Qt.FocusReason.OtherFocusReason)
-        if CoverFlow is not None:
-            if self.db_images.ignore_image_requests:
-                self.db_images.ignore_image_requests = False
-                self.db_images.dataChanged.emit()
-            self.cover_flow.setCurrentSlide(self.library_view.currentIndex().row())
-            self.cover_flow_syncing_enabled = True
-            QTimer.singleShot(500, self.cover_flow_do_sync)
+        if self.db_images.ignore_image_requests:
+            self.db_images.ignore_image_requests = False
+            self.db_images.dataChanged.emit()
+        self.cover_flow.setCurrentSlide(self.library_view.currentIndex().row())
+        self.cover_flow_syncing_enabled = True
+        QTimer.singleShot(500, self.cover_flow_do_sync)
         self.library_view.setCurrentIndex(
                 self.library_view.currentIndex())
         self.library_view.scroll_to_row(self.library_view.currentIndex().row())
 
     def cover_browser_hidden(self):
-        if CoverFlow is not None:
-            self.cover_flow_syncing_enabled = False
-            idx = self.library_view.model().index(self.cover_flow.currentSlide(), 0)
-            if idx.isValid():
-                sm = self.library_view.selectionModel()
-                sm.select(idx, QItemSelectionModel.SelectionFlag.ClearAndSelect|QItemSelectionModel.SelectionFlag.Rows)
-                self.library_view.setCurrentIndex(idx)
-                self.library_view.scroll_to_row(idx.row())
+        self.cover_flow_syncing_enabled = False
+        idx = self.library_view.model().index(self.cover_flow.currentSlide(), 0)
+        if idx.isValid():
+            sm = self.library_view.selectionModel()
+            sm.select(idx, QItemSelectionModel.SelectionFlag.ClearAndSelect|QItemSelectionModel.SelectionFlag.Rows)
+            self.library_view.setCurrentIndex(idx)
+            self.library_view.scroll_to_row(idx.row())
 
     def show_cover_browser(self):
         d = CBDialog(self, self.cover_flow)
@@ -487,7 +524,6 @@ def test():
     app = QApplication([])
     w = QMainWindow()
     cf = CoverFlow()
-    cf.resize(int(available_width()/1.5), available_height()-60)
     w.resize(cf.size()+QSize(30, 20))
     model = DummyImageList()
     cf.setImages(model)
@@ -496,7 +532,7 @@ def test():
 
     w.show()
     cf.setFocus(Qt.FocusReason.OtherFocusReason)
-    sys.exit(app.exec_())
+    sys.exit(app.exec())
 
 
 def main(args=sys.argv):
@@ -508,7 +544,6 @@ if __name__ == '__main__':
     app = QApplication([])
     w = QMainWindow()
     cf = CoverFlow()
-    cf.resize(int(available_width()/1.5), available_height()-60)
     w.resize(cf.size()+QSize(30, 20))
     path = sys.argv[1]
     model = FileSystemImages(sys.argv[1])
@@ -518,4 +553,4 @@ if __name__ == '__main__':
 
     w.show()
     cf.setFocus(Qt.FocusReason.OtherFocusReason)
-    sys.exit(app.exec_())
+    sys.exit(app.exec())
