@@ -5,37 +5,36 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import errno
 import functools
 import numbers
 import os
 import re
+import sys
 import time
 import traceback
 from collections import defaultdict, namedtuple
 from itertools import groupby
 from qt.core import (
-    QAbstractTableModel, QApplication, QColor, QDateTime, QFont, QFontMetrics, QIcon,
-    QImage, QModelIndex, QPainter, QPixmap, Qt, pyqtSignal,
+    QAbstractTableModel, QApplication, QColor, QFont, QFontMetrics, QIcon, QImage,
+    QModelIndex, QPainter, QPixmap, Qt, pyqtSignal,
 )
 
 from calibre import (
-    fit_image, force_unicode, human_readable, isbytestring, prepare_string_for_xml,
-    strftime,
+    fit_image, human_readable, isbytestring, prepare_string_for_xml, strftime,
 )
 from calibre.constants import DEBUG, config_dir, dark_link_color, filesystem_encoding
 from calibre.db.search import CONTAINS_MATCH, EQUALS_MATCH, REGEXP_MATCH, _match
+from calibre.db.utils import force_to_bool
 from calibre.ebooks.metadata import authors_to_string, fmt_sidx, string_to_authors
 from calibre.ebooks.metadata.book.formatter import SafeFormat
-from calibre.gui2 import error_dialog
+from calibre.gui2 import error_dialog, simple_excepthook
 from calibre.gui2.library import DEFAULT_SORT
-from calibre.library.caches import force_to_bool
 from calibre.library.coloring import color_row_key
 from calibre.library.save_to_disk import find_plugboard
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.config import device_prefs, prefs, tweaks
 from calibre.utils.date import (
-    UNDEFINED_DATE, as_local_time, dt_factory, is_date_undefined, qt_to_dt,
+    UNDEFINED_DATE, dt_factory, is_date_undefined, qt_from_dt, qt_to_dt,
 )
 from calibre.utils.icu import sort_key
 from calibre.utils.localization import calibre_langcode_to_name, ngettext
@@ -405,6 +404,8 @@ class BooksModel(QAbstractTableModel):  # {{{
     def refresh_rows(self, rows, current_row=-1):
         self._clear_caches()
         cc = self.columnCount(QModelIndex()) - 1
+        for r in rows:
+            self.db.new_api.clear_extra_files_cache(self.db.id(r))
         for first_row, last_row in group_numbers(rows):
             self.dataChanged.emit(self.index(first_row, 0), self.index(last_row, cc))
             if current_row >= 0 and first_row <= current_row <= last_row:
@@ -639,10 +640,11 @@ class BooksModel(QAbstractTableModel):  # {{{
                 return
             try:
                 data = self.get_book_display_info(idx)
-            except Exception:
-                import traceback
-                error_dialog(None, _('Unhandled error'), _(
-                    'Failed to read book data from calibre library. Click "Show details" for more information'), det_msg=traceback.format_exc(), show=True)
+            except Exception as e:
+                if sys.excepthook is simple_excepthook or sys.excepthook is sys.__excepthook__:
+                    return  # ignore failures during startup/shutdown
+                e.locking_violation_msg = _('Failed to read cover file for this book from the calibre library.')
+                raise
             else:
                 if emit_signal:
                     self.new_bookdisplay_data.emit(data)
@@ -917,7 +919,7 @@ class BooksModel(QAbstractTableModel):  # {{{
             elif dt == 'datetime':
                 def func(idx):
                     val = fffunc(field_obj, idfunc(idx), default_value=UNDEFINED_DATE)
-                    return None if is_date_undefined(val) else QDateTime(as_local_time(val))
+                    return None if is_date_undefined(val) else qt_from_dt(val)
             elif dt == 'rating':
                 rating_fields[field] = m['display'].get('allow_half_stars', False)
 
@@ -1255,13 +1257,10 @@ class BooksModel(QAbstractTableModel):  # {{{
                 return self._set_data(index, value)
             except OSError as err:
                 import traceback
-                if getattr(err, 'errno', None) == errno.EACCES:  # Permission denied
-                    fname = getattr(err, 'filename', None)
-                    p = 'Locked file: %s\n\n'%force_unicode(fname if fname else '')
-                    error_dialog(get_gui(), _('Permission denied'),
-                            _('Could not change the on disk location of this'
-                                ' book. Is it open in another program?'),
-                            det_msg=p+force_unicode(traceback.format_exc()), show=True)
+                traceback.print_exc()
+                det_msg = traceback.format_exc()
+                gui = get_gui()
+                if gui.show_possible_sharing_violation(err, det_msg):
                     return False
                 error_dialog(get_gui(), _('Failed to set data'),
                         _('Could not set data, click "Show details" to see why.'),
@@ -1594,6 +1593,8 @@ class DeviceBooksModel(BooksModel):  # {{{
         self.search(self.last_search, reset)
 
     def sort(self, col, order, reset=True):
+        if not isinstance(order, Qt.SortOrder):
+            order = Qt.SortOrder.AscendingOrder if order else Qt.SortOrder.DescendingOrder
         descending = order != Qt.SortOrder.AscendingOrder
         cname = self.column_map[col]
 
