@@ -15,13 +15,12 @@ from qt.core import (
 )
 
 from calibre.ebooks.metadata import title_sort
-from calibre.gui2 import config, error_dialog, gprefs, question_dialog
+from calibre.gui2 import config, error_dialog, gprefs, question_dialog, warning_dialog
 from calibre.gui2.dialogs.edit_authors_dialog import EditAuthorsDialog
 from calibre.gui2.dialogs.tag_categories import TagCategories
 from calibre.gui2.dialogs.tag_list_editor import TagListEditor
 from calibre.gui2.tag_browser.view import TagsView
 from calibre.gui2.widgets import HistoryLineEdit
-from calibre.library.field_metadata import category_icon_map
 from calibre.startup import connect_lambda
 from calibre.utils.icu import sort_key
 from calibre.utils.localization import ngettext
@@ -34,46 +33,89 @@ class TagBrowserMixin:  # {{{
         pass
 
     def populate_tb_manage_menu(self, db):
-        self.populate_manage_categories_menu(db, self.alter_tb.manage_menu)
+        self.populate_manage_categories_menu(db, self.alter_tb.manage_menu, add_column_items=False)
 
-    def populate_manage_categories_menu(self, db, menu):
+    def populate_manage_categories_menu(self, db, menu, add_column_items=False):
         from calibre.db.categories import find_categories
+        fm = db.new_api.field_metadata
+
+        # Get the custom categories
+        cust_cats = [x[0] for x in find_categories(fm) if fm.is_custom_field(x[0])]
+        cust_cats = [c for c in cust_cats if fm[c]['datatype'] != 'composite']
+
         m = menu
         m.clear()
-        for text, func, args, cat_name in (
-             (_('Authors'),
-                        self.do_author_sort_edit, (self, None), 'authors'),
-             (ngettext('Series', 'Series', 2),
-                        self.do_tags_list_edit, (None, 'series'), 'series'),
-             (_('Publishers'),
-                        self.do_tags_list_edit, (None, 'publisher'), 'publisher'),
-             (_('Tags'),
-                        self.do_tags_list_edit, (None, 'tags'), 'tags'),
-             (_('User categories'),
-                        self.do_edit_user_categories, (None,), 'user:'),
-             (_('Saved searches'),
-                        self.do_saved_search_edit, (None,), 'search')
-            ):
-            m.addAction(QIcon.ic(category_icon_map[cat_name]), text,
-                    partial(func, *args))
-        fm = db.new_api.field_metadata
-        categories = [x[0] for x in find_categories(fm) if fm.is_custom_field(x[0])]
-        categories = [c for c in categories if fm[c]['datatype'] != 'composite']
-        if categories:
-            if len(categories) > 5:
-                m = m.addMenu(_('Custom columns'))
-            else:
-                m.addSeparator()
 
-            def cat_key(x):
-                try:
-                    return fm[x]['name']
-                except Exception:
-                    return ''
-            for cat in sorted(categories, key=cat_key):
-                name = cat_key(cat)
-                if name:
-                    m.addAction(name, partial(self.do_tags_list_edit, None, cat))
+        def cat_display_name(x):
+            try:
+                if x == 'series':
+                    return ngettext('Series', 'Series', 2)
+                if x == 'user:':
+                    return _('User categories')
+                return fm[x]['name']
+            except Exception:
+                return ''
+
+        def get_icon(cat_name):
+            icon = self.tags_view.model().category_custom_icons.get(cat_name, None)
+            if not icon:
+                from calibre.library.field_metadata import category_icon_map
+                icon = QIcon.ic(category_icon_map.get(cat_name) or category_icon_map['custom:'])
+            return icon
+
+        def menu_func(cat_name, item):
+            if cat_name == 'authors':
+                return partial(self.do_author_sort_edit, self,
+                               None if item is None else db.new_api.get_item_id('authors', item),
+                               select_sort=False)
+            elif cat_name == 'user:':
+                return partial(self.do_edit_user_categories, None)
+            elif cat_name == 'search':
+                return partial(self.do_saved_search_edit, None)
+            else:
+                return partial(self.do_tags_list_edit, item, cat_name)
+
+        # Check if the current cell is a category. If so, show an action for it
+        # and its items on the current book.
+        current_cat = None
+        idx = self.library_view.currentIndex() if add_column_items else None
+        if idx is not None and idx.isValid():
+            col = idx.column()
+            model = self.library_view.model()
+            if col in range(0, len(model.column_map)):
+                current_cat = model.column_map[col]
+                if current_cat in ('authors', 'series', 'publisher', 'tags') or current_cat in cust_cats:
+                    cdn = cat_display_name(current_cat) or current_cat
+                    m.addAction(get_icon(current_cat), cdn, menu_func(current_cat, None))
+                    proxy_md = db.new_api.get_proxy_metadata(db.id(idx.row()))
+                    items = proxy_md.get(current_cat)
+                    if isinstance(items, str):
+                        items = list((items,))
+                    if items:
+                        items_title = _('{category} for current book').format(category=cdn)
+                        if len(items) > 4:
+                            im = QMenu(items_title, m)
+                            im.setIcon(get_icon(current_cat))
+                            m.addMenu(im)
+                        else:
+                            m.addSection(items_title)
+                            im = m
+                        for item in sorted(items, key=sort_key):
+                            im.addAction(get_icon(current_cat), item, menu_func(current_cat, item))
+                    m.addSection(_('Other categories'))
+        # Standard columns
+        for key in ('authors', 'series', 'publisher', 'tags', 'user:', 'search'):
+            if cat_display_name(key) and key != current_cat:
+                m.addAction(get_icon(key), cat_display_name(key), menu_func(key, None))
+
+        # Custom columns
+        if cust_cats:
+            if len(cust_cats) > 5:
+                m = m.addMenu(_('Custom categories'))
+            for cat in sorted(cust_cats, key=lambda v: sort_key(cat_display_name(v))):
+                if cat == current_cat:
+                    continue
+                m.addAction(get_icon(cat), cat_display_name(cat), menu_func(cat, None))
 
     def init_tag_browser_mixin(self, db):
         self.library_view.model().count_changed_signal.connect(self.tags_view.recount_with_position_based_index)
@@ -88,7 +130,7 @@ class TagBrowserMixin:  # {{{
         self.tags_view.saved_search_edit.connect(self.do_saved_search_edit)
         self.tags_view.rebuild_saved_searches.connect(self.do_rebuild_saved_searches)
         self.tags_view.author_sort_edit.connect(self.do_author_sort_edit)
-        self.tags_view.tag_item_renamed.connect(self.do_tag_item_renamed)
+        self.tags_view.tag_item_renamed.connect(self.do_field_item_value_changed)
         self.tags_view.search_item_renamed.connect(self.saved_searches_changed)
         self.tags_view.drag_drop_finished.connect(self.drag_drop_finished)
         self.tags_view.restriction_error.connect(self.do_restriction_error,
@@ -115,7 +157,6 @@ class TagBrowserMixin:  # {{{
         self.library_view.model().research(reset=False)
         self.library_view.current_id = current_row_id # the setter checks for None
 
-
     def do_restriction_error(self, e):
         error_dialog(self.tags_view, _('Invalid search restriction'),
                          _('The current search restriction is invalid'),
@@ -128,6 +169,18 @@ class TagBrowserMixin:  # {{{
         opportunity to edit the name.
         '''
         db = self.library_view.model().db
+        m = self.tags_view.model()
+        # Can't add an unnamed pref when empty categories are hidden. There is no
+        # way for the user to see/edit it.
+        if new_category_name is None and m.prefs['tag_browser_hide_empty_categories']:
+            error_dialog(self.tags_view, _('Cannot add subcategory to category'),
+                    _("The option 'Preferences -> Look & feel -> Tag browser -> "
+                      "Hide empty categories' is enabled, preventing the creation "
+                      "of new empty user subcategories because they won't be "
+                      "displayed. Either change the option or use the 'Manage "
+                      "Categories' dialog to add the subcategories."),
+                    show=True)
+            return
         user_cats = db.new_api.pref('user_categories', {})
 
         # Ensure that the temporary name we will use is not already there
@@ -148,7 +201,6 @@ class TagBrowserMixin:  # {{{
         db.new_api.set_pref('user_categories', user_cats)
         self.tags_view.recount()
         db.new_api.clear_search_caches()
-        m = self.tags_view.model()
         idx = m.index_for_path(m.find_category_node('@' + new_cat))
         self.tags_view.show_item_at_index(idx)
         # Open the editor on the new item to rename it
@@ -259,8 +311,22 @@ class TagBrowserMixin:  # {{{
         db.new_api.clear_search_caches()
         self.user_categories_edited()
 
+    # Keep this for compatibility. It isn't used here but could be used in a plugin
     def get_book_ids(self, use_virtual_library, db, category):
-        book_ids = None if not use_virtual_library else self.tags_view.model().get_book_ids_to_use()
+        return self.get_book_ids_in_vl_or_selection(
+            ('virtual_library' if use_virtual_library else None), db, category)
+
+    def get_book_ids_in_vl_or_selection(self, use_what, db, category):
+        if use_what is None:
+            book_ids = None
+        elif use_what == 'virtual_library':
+            book_ids = self.tags_view.model().get_book_ids_to_use()
+        else:
+            book_ids = self.library_view.get_selected_ids()
+            if not book_ids:
+                warning_dialog(self.tags_view, _('No books selected'),
+                               _('No books are selected. Showing all items.'), show=True)
+                book_ids = None
         data = db.new_api.get_categories(book_ids=book_ids)
         if category in data:
             result = [(t.id, t.original_name, t.count) for t in data[category] if t.count > 0]
@@ -284,7 +350,7 @@ class TagBrowserMixin:  # {{{
         d = TagListEditor(self, category=category,
                           cat_name=db.field_metadata[category]['name'],
                           tag_to_match=tag,
-                          get_book_ids=partial(self.get_book_ids, db=db, category=category),
+                          get_book_ids=partial(self.get_book_ids_in_vl_or_selection, db=db, category=category),
                           sorter=key, ttm_is_first_letter=is_first_letter,
                           fm=db.field_metadata[category],
                           link_map=db.new_api.get_link_map(category))
@@ -310,7 +376,7 @@ class TagBrowserMixin:  # {{{
                 db.new_api.set_link_map(category, d.links)
 
                 # Clean up the library view
-                self.do_tag_item_renamed()
+                self.do_field_item_value_changed()
                 self.tags_view.recount()
 
     def do_tag_item_delete(self, category, item_id, orig_name,
@@ -362,7 +428,7 @@ class TagBrowserMixin:  # {{{
             m.delete_item_from_all_user_categories(orig_name, category)
 
         # Clean up the library view
-        self.do_tag_item_renamed()
+        self.do_field_item_value_changed()
         self.tags_view.recount()
 
     def apply_tag_to_selected(self, field_name, item_name, remove):
@@ -439,8 +505,9 @@ class TagBrowserMixin:  # {{{
         d = EnumValuesEdit(parent, db, key)
         d.exec()
 
-    def do_tag_item_renamed(self):
-        # Clean up library view and search
+    def do_field_item_value_changed(self):
+        # Clean up library view and search, which also cleans up book details
+
         # get information to redo the selection
         rows = [r.row() for r in
                 self.library_view.selectionModel().selectedRows()]
@@ -451,6 +518,7 @@ class TagBrowserMixin:  # {{{
         m.research()
         self.library_view.select_rows(ids)
         # refreshing the tags view happens at the emit()/call() site
+    do_tag_item_renamed = do_field_item_value_changed  # alias for backcompat
 
     def do_author_sort_edit(self, parent, id_, select_sort=True,
                             select_link=False, is_first_letter=False,
@@ -458,11 +526,10 @@ class TagBrowserMixin:  # {{{
         '''
         Open the manage authors dialog
         '''
-
         db = self.library_view.model().db
-        get_authors_func = partial(self.get_book_ids, db=db, category='authors')
+        get_authors_func = partial(self.get_book_ids_in_vl_or_selection, db=db, category='authors')
         if lookup_author:
-            for t in get_authors_func(use_virtual_library=False):
+            for t in get_authors_func(None):
                 if t[1] == id_:
                     id_ = t[0]
                     break
@@ -484,7 +551,8 @@ class TagBrowserMixin:  # {{{
                 sort_map = {id_map.get(author_id, author_id):new_sort for author_id, old_author, new_author, new_sort, new_link in editor.result}
                 affected_books |= db.set_sort_for_authors(sort_map)
                 self.library_view.model().refresh_ids(affected_books, current_row=self.library_view.currentIndex().row())
-                self.tags_view.recount()
+            self.do_field_item_value_changed()
+            self.tags_view.recount()
 
     def drag_drop_finished(self, ids):
         self.library_view.model().refresh_ids(ids)
@@ -566,15 +634,23 @@ class TagBrowserBar(QWidget):  # {{{
         self.item_search.initialize('tag_browser_search')
         self.item_search.completer().setCaseSensitivity(Qt.CaseSensitivity.CaseSensitive)
         self.item_search.setToolTip(
-            '<p>' +_(
-                'Search for items. If the text begins with equals (=) the search is '
-                'exact match, otherwise it is "contains" finding items containing '
-                'the text anywhere in the item name. Both exact and contains '
-                'searches ignore case. You can limit the search to particular '
-                'categories using syntax similar to search. For example, '
-                'tags:foo will find foo in any tag, but not in authors etc. Entering '
-                '*foo will collapse all categories then showing only those categories '
-                'with items containing the text "foo"') + '</p>')
+            _('<p>'
+                'Search for items in the Tag browser. If the search text begins '
+                'with an equals sign (=) then the search is "equals", otherwise '
+                'it is "contains". Both the equals and contains searches ignore '
+                'case. If the preference <em>Preferences -> Searching -> Unaccented '
+                'characters match accented characters ...</em> is checked then a '
+                '<em>Character variant search</em> is used, where characters '
+                'match regardless of accents, and punctuation is significant. See '
+                '<em>The search interface</em> in the calibre manual for more explanation.'
+            '</p><p>'
+                'You can limit the search to particular categories using syntax '
+                "similar to calibre's <em>Search</em>. For example, tags:foo will "
+                'find foo in tags but not in authors etc.'
+            '</p><p>'
+                'Entering *foo will collapse all categories before doing the '
+                'search.'
+            '</p>'))
         ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser find box',
