@@ -17,22 +17,23 @@ import sys
 import textwrap
 import time
 from collections import OrderedDict, deque
+from functools import partial
 from io import BytesIO
 from qt.core import (
     QAction, QApplication, QDialog, QFont, QIcon, QMenu, QSystemTrayIcon, Qt, QTimer,
-    QUrl, pyqtSignal
+    QUrl, pyqtSignal,
 )
 
 from calibre import detect_ncpus, force_unicode, prints
 from calibre.constants import (
-    DEBUG, __appname__, config_dir, filesystem_encoding, ismacos, iswindows
+    DEBUG, __appname__, config_dir, filesystem_encoding, ismacos, iswindows,
 )
 from calibre.customize import PluginInstallationType
 from calibre.customize.ui import available_store_plugins, interface_actions
 from calibre.db.legacy import LibraryDatabase
 from calibre.gui2 import (
     Dispatcher, GetMetadata, config, error_dialog, gprefs, info_dialog,
-    max_available_height, open_url, question_dialog, warning_dialog
+    max_available_height, open_url, question_dialog, timed_print, warning_dialog,
 )
 from calibre.gui2.auto_add import AutoAdder
 from calibre.gui2.changes import handle_changes
@@ -41,6 +42,7 @@ from calibre.gui2.device import DeviceMixin
 from calibre.gui2.dialogs.message_box import JobError
 from calibre.gui2.ebook_download import EbookDownloadMixin
 from calibre.gui2.email import EmailMixin
+from calibre.gui2.extra_files_watcher import ExtraFilesWatcher
 from calibre.gui2.init import LayoutMixin, LibraryViewMixin
 from calibre.gui2.job_indicator import Pointer
 from calibre.gui2.jobs import JobManager, JobsButton, JobsDialog
@@ -59,6 +61,7 @@ from calibre.library import current_library_name
 from calibre.srv.library_broker import GuiLibraryBroker, db_matches
 from calibre.utils.config import dynamic, prefs
 from calibre.utils.ipc.pool import Pool
+from calibre.utils.resources import get_image_path as I, get_path as P
 from polyglot.builtins import string_or_bytes
 from polyglot.queue import Empty, Queue
 
@@ -77,12 +80,12 @@ def add_quick_start_guide(library_view, refresh_cover_browser=None):
     gprefs['quick_start_guide_added'] = True
     imgbuf = BytesIO(calibre_cover2(_('Quick Start Guide'), ''))
     try:
-        with lopen(P('quick_start/%s.epub' % l), 'rb') as src:
+        with open(P('quick_start/%s.epub' % l), 'rb') as src:
             buf = BytesIO(src.read())
     except OSError as err:
         if err.errno != errno.ENOENT:
             raise
-        with lopen(P('quick_start/eng.epub'), 'rb') as src:
+        with open(P('quick_start/eng.epub'), 'rb') as src:
             buf = BytesIO(src.read())
     safe_replace(buf, 'images/cover.jpg', imgbuf)
     buf.seek(0)
@@ -116,7 +119,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
     def __init__(self, opts, parent=None, gui_debug=None):
         MainWindow.__init__(self, opts, parent=parent, disable_automatic_gc=True)
+        self.setVisible(False)
         self.setWindowIcon(QApplication.instance().windowIcon())
+        self.extra_files_watcher = ExtraFilesWatcher(self)
         self.jobs_pointer = Pointer(self)
         self.proceed_requested.connect(self.do_proceed,
                 type=Qt.ConnectionType.QueuedConnection)
@@ -369,7 +374,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         # ########################## Cover Flow ################################
 
-        CoverFlowMixin.init_cover_flow_mixin(self)
+        CoverFlowMixin.__init__(self)
 
         self._calculated_available_height = min(max_available_height()-15,
                 self.height())
@@ -388,9 +393,12 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         if config['autolaunch_server']:
             self.start_content_server()
-
-        if show_gui:
-            self.show()
+        do_hide_windows = False
+        if self.system_tray_icon is not None and self.system_tray_icon.isVisible() and opts.start_in_tray:
+            do_hide_windows = True
+            show_gui = False
+            setattr(self, '__systray_minimized', True)
+        QTimer.singleShot(0, partial(self.post_initialize_actions, show_gui, do_hide_windows))
         self.read_settings()
 
         self.finalize_layout()
@@ -410,41 +418,54 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         register_keyboard_shortcuts()
         self.keyboard.finalize()
-        if self.system_tray_icon is not None and self.system_tray_icon.isVisible() and opts.start_in_tray:
-            self.hide_windows()
         self.auto_adder = AutoAdder(gprefs['auto_add_path'], self)
 
         self.listener = Listener(parent=self)
         self.listener.message_received.connect(self.message_from_another_instance)
-        QTimer.singleShot(0, self.listener.start_listening)
-
-        # Collect cycles now
-        gc.collect()
 
         QApplication.instance().shutdown_signal_received.connect(self.quit)
         if show_gui and self.gui_debug is not None:
             QTimer.singleShot(10, self.show_gui_debug_msg)
 
         self.iactions['Connect Share'].check_smartdevice_menus()
-        QTimer.singleShot(1, self.start_smartdevice)
         QTimer.singleShot(100, self.update_toggle_to_tray_action)
 
+    def post_initialize_actions(self, show_gui, do_hide_windows):
+        # Various post-initialization actions after an event loop tick
+
+        # Collect cycles now
+        gc.collect()
+
+        self.listener.start_listening()
+        if do_hide_windows:
+            self.hide_windows()
+        if show_gui:
+            timed_print('GUI main window shown')
+            self.show()
+        # Force repaint of the book details splitter because it otherwise ends
+        # up with the wrong size. I don't know why.
+        self.bd_splitter.repaint()
+
         # Once the gui is initialized we can restore the quickview state
         # The same thing will be true for any action-based operation with a
         # layout button. We need to let a book be selected in the book list
         # before initializing quickview, so run it after an event loop tick
         QTimer.singleShot(0, self.start_quickview)
-        # Force repaint of the book details splitter because it otherwise ends
-        # up with the wrong size. I don't know why.
-        QTimer.singleShot(0, self.bd_splitter.repaint)
+
+        # Start the smartdevice later so that the network time doesn't affect
+        # the gui repaint debouncing. Wait 3 seconds before starting to be sure
+        # that all other initialization (plugins etc) has completed. Yes, 3
+        # seconds is an arbitrary value and probably too long, but it will do
+        # until the underlying structure changes to make it unnecessary.
+        QTimer.singleShot(3000, self.start_smartdevice)
 
     def start_quickview(self):
         from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
         qv = get_quickview_action_plugin()
         if qv:
-            if DEBUG:
-                prints('Starting QuickView')
+            timed_print('QuickView starting')
             qv.qv_button.restore_state()
+            timed_print('QuickView started')
         self.save_layout_state()
         self.focus_library_view()
 
@@ -474,13 +495,15 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.focus_current_view()
 
     def start_smartdevice(self):
+        timed_print('Starting the smartdevice driver')
         message = None
         if self.device_manager.get_option('smartdevice', 'autostart'):
             try:
                 message = self.device_manager.start_plugin('smartdevice')
-            except:
-                message = 'start smartdevice unknown exception'
-                prints(message)
+                timed_print('Finished starting smartdevice')
+            except Exception as e:
+                message = str(e)
+                timed_print(f'Starting smartdevice driver failed: {message}')
                 import traceback
                 traceback.print_exc()
         if message:
@@ -625,7 +648,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.tags_view.recount()
 
     def handle_cli_args(self, args):
-        from urllib.parse import unquote, urlparse, parse_qs
+        from urllib.parse import parse_qs, unquote, urlparse
         if isinstance(args, string_or_bytes):
             args = [args]
         files, urls = [], []
@@ -674,11 +697,84 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 return bytes.fromhex(x[6:]).decode('utf-8')
             return x
 
+        def get_virtual_library(query):
+            vl = None
+            if query.get('encoded_virtual_library'):
+                vl = bytes.fromhex(query.get('encoded_virtual_library')[0]).decode('utf-8')
+            elif query.get('virtual_library'):
+                vl = query.get('virtual_library')[0]
+            if vl == '-':
+                vl = None
+            return vl
+
         if action == 'switch-library':
             library_id = decode_library_id(posixpath.basename(path))
             library_path = self.library_broker.path_for_library_id(library_id)
             if not db_matches(self.current_db, library_id, library_path):
                 self.library_moved(library_path)
+        elif action == 'book-details':
+            parts = tuple(filter(None, path.split('/')))
+            if len(parts) != 2:
+                return
+            library_id, book_id = parts
+            library_id = decode_library_id(library_id)
+            library_path = self.library_broker.path_for_library_id(library_id)
+            if library_path is None:
+                prints('Ignoring unknown library id', library_id, file=sys.stderr)
+                return
+            try:
+                book_id = int(book_id)
+            except Exception:
+                prints('Ignoring invalid book id', book_id, file=sys.stderr)
+                return
+            details = self.iactions['Show Book Details']
+            details.show_book_info(library_id=library_id, library_path=library_path, book_id=book_id)
+        elif action == 'show-note':
+            parts = tuple(filter(None, path.split('/')))
+            if len(parts) != 3:
+                return
+            library_id, field, itemx = parts
+            library_id = decode_library_id(library_id)
+            library_path = self.library_broker.path_for_library_id(library_id)
+            if library_path is None:
+                prints('Ignoring unknown library id', library_id, file=sys.stderr)
+                return
+            if field.startswith('_'):
+                field = '#' + field[1:]
+            item_id = item_val = None
+            if itemx.startswith('id_'):
+                try:
+                    item_id = int(itemx[3:])
+                except Exception:
+                    prints('Ignoring invalid item id', itemx, file=sys.stderr)
+                    return
+            elif itemx.startswith('hex_'):
+                try:
+                    item_val = bytes.fromhex(itemx[4:]).decode('utf-8')
+                except Exception:
+                    prints('Ignoring invalid item hexval', itemx, file=sys.stderr)
+                    return
+            elif itemx.startswith('val_'):
+                item_val = itemx[4:]
+            else:
+                prints('Ignoring invalid item hexval', itemx, file=sys.stderr)
+                return
+
+            def doit():
+                nonlocal item_id, item_val
+                db = self.current_db.new_api
+                if item_id is None:
+                    item_id = db.get_item_id(field, item_val)
+                    if item_id is None:
+                        prints('The item named:', item_val, 'was not found', file=sys.stderr)
+                        return
+                if db.notes_for(field, item_id):
+                    from calibre.gui2.dialogs.show_category_note import ShowNoteDialog
+                    ShowNoteDialog(field, item_id, db, parent=self).show()
+                else:
+                    prints(f'No notes available for {field}:{itemx}', file=sys.stderr)
+
+            self.perform_url_action(library_id, library_path, doit)
         elif action == 'show-book':
             parts = tuple(filter(None, path.split('/')))
             if len(parts) != 2:
@@ -693,9 +789,16 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             library_path = self.library_broker.path_for_library_id(library_id)
             if library_path is None:
                 return
+            vl = get_virtual_library(query)
 
             def doit():
+                # To maintain compatibility, don't change the VL if it isn't specified.
+                if vl is not None and vl != '_':
+                    self.apply_virtual_library(vl)
                 rows = self.library_view.select_rows((book_id,))
+                if not rows:
+                    self.search.set_search_string('')
+                    rows = self.library_view.select_rows((book_id,))
                 db = self.current_db
                 if not rows and (db.data.get_base_restriction_name() or db.data.get_search_restriction_name()):
                     self.apply_virtual_library()
@@ -742,13 +845,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 if sq:
                     sq = sq[0]
             sq = sq or ''
-            vl = None
-            if query.get('encoded_virtual_library'):
-                vl = bytes.fromhex(query.get('encoded_virtual_library')[0]).decode('utf-8')
-            elif query.get('virtual_library'):
-                vl = query.get('virtual_library')[0]
-            if vl == '-':
-                vl = None
+            vl = get_virtual_library(query)
 
             def doit():
                 if vl != '_':
@@ -870,7 +967,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                             )
                     if repair:
                         from calibre.gui2.dialogs.restore_library import (
-                            repair_library_at
+                            repair_library_at,
                         )
                         if repair_library_at(newloc, parent=self):
                             db = LibraryDatabase(newloc, default_prefs=default_prefs)
@@ -885,6 +982,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     import traceback
                     traceback.print_exc()
             self.library_path = newloc
+            self.extra_files_watcher.clear()
             prefs['library_path'] = self.library_path
             self.book_on_device(None, reset=True)
             db.set_book_on_device_func(self.book_on_device)
@@ -1112,14 +1210,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             if not question_dialog(self, _('Library updates waiting'), msg):
                 return False
 
-        from calibre.db.delete_service import has_jobs
-        if has_jobs():
-            msg = _('Some deleted books are still being moved to the recycle '
-                    'bin, if you quit now, they will be left behind. Are you '
-                    'sure you want to quit?')
-            if not question_dialog(self, _('Active jobs'), msg):
-                return False
-
         return True
 
     def shutdown(self, write_settings=True):
@@ -1127,6 +1217,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.shutdown_started.emit()
         self.show_shutdown_message()
         self.server_change_notification_timer.stop()
+        self.extra_files_watcher.clear()
         try:
             self.event_in_db.disconnect()
         except Exception:
@@ -1200,8 +1291,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self._spare_pool.shutdown()
         from calibre.scraper.simple import cleanup_overseers
         wait_for_cleanup = cleanup_overseers()
-        from calibre.db.delete_service import shutdown
-        shutdown()
         from calibre.live import async_stop_worker
         wait_for_stop = async_stop_worker()
         time.sleep(2)
