@@ -3,26 +3,29 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''
 Miscellaneous widgets used in the GUI
 '''
-import re, os
+import os
+import re
+from qt.core import (
+    QAction, QApplication, QClipboard, QColor, QComboBox, QCompleter, QCursor, QEvent,
+    QFont, QGraphicsScene, QGraphicsView, QIcon, QKeySequence, QLabel, QLineEdit,
+    QListWidget, QListWidgetItem, QMenu, QPageSize, QPainter, QPalette, QPen, QPixmap,
+    QPrinter, QRect, QSize, QSplitter, QSplitterHandle, QStringListModel,
+    QSyntaxHighlighter, Qt, QTextCharFormat, QTimer, QToolButton, QWidget, pyqtSignal,
+)
 
-from qt.core import (QIcon, QFont, QLabel, QListWidget, QAction, QEvent,
-        QListWidgetItem, QTextCharFormat, QApplication, QSyntaxHighlighter,
-        QCursor, QColor, QWidget, QPixmap, QSplitterHandle, QToolButton,
-        Qt, pyqtSignal, QRegExp, QSize, QSplitter, QPainter, QPageSize, QPrinter,
-        QLineEdit, QComboBox, QPen, QGraphicsScene, QMenu, QStringListModel, QKeySequence,
-        QCompleter, QTimer, QRect, QGraphicsView, QPagedPaintDevice, QPalette, QClipboard)
-
-from calibre.constants import iswindows, ismacos
-from calibre.gui2 import (error_dialog, pixmap_to_data, gprefs,
-        warning_dialog)
-from calibre.gui2.filename_pattern_ui import Ui_Form
-from calibre import fit_image, strftime, force_unicode
+from calibre import fit_image, force_unicode, strftime
+from calibre.constants import ismacos, iswindows
 from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.utils.config import prefs, XMLConfig
+from calibre.gui2 import error_dialog, gprefs, pixmap_to_data, warning_dialog
+from calibre.gui2.dnd import (
+    DownloadDialog, dnd_get_files, dnd_get_image, dnd_get_local_image_and_pixmap,
+    dnd_has_extension, dnd_has_image, image_extensions,
+)
+from calibre.gui2.filename_pattern_ui import Ui_Form
 from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
-from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
-    image_extensions, dnd_has_extension, dnd_get_local_image_and_pixmap, DownloadDialog)
-from calibre.utils.localization import localize_user_manual_link
+from calibre.startup import connect_lambda
+from calibre.utils.config import XMLConfig, prefs
+from calibre.utils.localization import _, localize_user_manual_link
 from polyglot.builtins import native_string_type
 
 history = XMLConfig('history')
@@ -119,7 +122,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
             pat = self.pattern()
         except Exception as err:
             error_dialog(self, _('Invalid regular expression'),
-                         _('Invalid regular expression: %s')%err).exec_()
+                         _('Invalid regular expression: %s')%err).exec()
             return
         mi = metadata_from_filename(fname, pat)
         if mi.title:
@@ -254,7 +257,7 @@ class ImageDropMixin:  # {{{
                 d.start_download()
                 if d.err is None:
                     pmap = QPixmap()
-                    with lopen(d.fpath, 'rb') as f:
+                    with open(d.fpath, 'rb') as f:
                         data = f.read()
                     pmap.loadFromData(data)
                     if not pmap.isNull():
@@ -275,8 +278,8 @@ class ImageDropMixin:  # {{{
 
     def build_context_menu(self):
         cm = QMenu(self)
-        paste = cm.addAction(_('Paste cover'))
-        copy = cm.addAction(_('Copy cover'))
+        paste = cm.addAction(QIcon.ic('edit-paste.png'), _('Paste cover'))
+        copy = cm.addAction(QIcon.ic('edit-copy.png'), _('Copy cover'))
         if not QApplication.instance().clipboard().mimeData().hasImage():
             paste.setEnabled(False)
         copy.triggered.connect(self.copy_to_clipboard)
@@ -284,7 +287,7 @@ class ImageDropMixin:  # {{{
         return cm
 
     def contextMenuEvent(self, ev):
-        self.build_context_menu().exec_(ev.globalPos())
+        self.build_context_menu().exec(ev.globalPos())
 
     def copy_to_clipboard(self):
         QApplication.instance().clipboard().setPixmap(self.get_pixmap())
@@ -298,6 +301,8 @@ class ImageDropMixin:  # {{{
             self.set_pixmap(pmap)
             self.cover_changed.emit(
                     pixmap_to_data(pmap, format='PNG'))
+            return True
+        return False
 # }}}
 
 
@@ -320,6 +325,7 @@ class ImageView(QWidget, ImageDropMixin):
 
     BORDER_WIDTH = 1
     cover_changed = pyqtSignal(object)
+    draw_empty_border = False
 
     def __init__(self, parent=None, show_size_pref_name=None, default_show_size=False):
         QWidget.__init__(self, parent)
@@ -363,7 +369,15 @@ class ImageView(QWidget, ImageDropMixin):
     def paintEvent(self, event):
         QWidget.paintEvent(self, event)
         pmap = self._pixmap
+        p = QPainter(self)
+        p.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
         if pmap.isNull():
+            if self.draw_empty_border:
+                pen = QPen()
+                pen.setWidth(self.BORDER_WIDTH)
+                p.setPen(pen)
+                p.drawRect(self.rect())
+                p.end()
             return
         w, h = pmap.width(), pmap.height()
         ow, oh = w, h
@@ -376,8 +390,6 @@ class ImageView(QWidget, ImageDropMixin):
         x = int(abs(cw - w)/2)
         y = int(abs(ch - h)/2)
         target = QRect(x, y, w, h)
-        p = QPainter(self)
-        p.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
         p.drawPixmap(target, pmap)
         if self.draw_border:
             pen = QPen()
@@ -386,7 +398,6 @@ class ImageView(QWidget, ImageDropMixin):
             p.drawRect(target)
         if self.show_size:
             draw_size(p, target, ow, oh)
-        p.end()
 # }}}
 
 
@@ -490,27 +501,36 @@ class LineEditECM:  # {{{
         menu = self.createStandardContextMenu()
         menu.addSeparator()
         self.create_change_case_menu(menu)
-        menu.exec_(event.globalPos())
+        if callable(getattr(self, 'add_items_to_context_menu', None)):
+            menu = self.add_items_to_context_menu(self, menu)
+        menu.exec(event.globalPos())
+
+    def modify_case_operation(self, func):
+        has_selection = self.hasSelectedText()
+        text = self.selectedText() if has_selection else self.text()
+        ntext = func(text)
+        if ntext != text:
+            self.insert(ntext) if has_selection else self.setText(ntext)
 
     def upper_case(self):
         from calibre.utils.icu import upper
-        self.setText(upper(str(self.text())))
+        self.modify_case_operation(upper)
 
     def lower_case(self):
         from calibre.utils.icu import lower
-        self.setText(lower(str(self.text())))
+        self.modify_case_operation(lower)
 
     def swap_case(self):
         from calibre.utils.icu import swapcase
-        self.setText(swapcase(str(self.text())))
+        self.modify_case_operation(swapcase)
 
     def title_case(self):
         from calibre.utils.titlecase import titlecase
-        self.setText(titlecase(str(self.text())))
+        self.modify_case_operation(titlecase)
 
     def capitalize(self):
         from calibre.utils.icu import capitalize
-        self.setText(capitalize(str(self.text())))
+        self.modify_case_operation(capitalize)
 
 # }}}
 
@@ -534,6 +554,45 @@ class EnLineEdit(LineEditECM, QLineEdit):  # {{{
 # }}}
 
 
+# LineEditIndicators {{{
+
+def setup_status_actions(self: QLineEdit):
+    self.status_actions = (
+        self.addAction(QIcon.ic('ok.png'), QLineEdit.ActionPosition.TrailingPosition),
+        self.addAction(QIcon.ic('dialog_error.png'), QLineEdit.ActionPosition.TrailingPosition))
+    self.status_actions[0].setVisible(False)
+    self.status_actions[1].setVisible(False)
+
+
+def stylesheet_for_lineedit(ok, selector='QLineEdit') -> str:
+    if ok is None:
+        return ''
+    col = '#50c878' if ok else '#FF2400'
+    return f'{selector} {{ border: 2px solid {col}; border-radius: 3px }}'
+
+
+def update_status_actions(self: QLineEdit, ok, tooltip: str = ''):
+    self.status_actions[0].setVisible(bool(ok))
+    self.status_actions[1].setVisible(not ok)
+    if ok:
+        self.status_actions[0].setToolTip(tooltip)
+    elif ok is None:
+        self.status_actions[1].setVisible(False)
+    else:
+        self.status_actions[1].setToolTip(tooltip)
+    self.setStyleSheet(stylesheet_for_lineedit(ok))
+
+
+class LineEditIndicators:
+
+    def setup_status_actions(self):
+        setup_status_actions(self)
+
+    def update_status_actions(self, ok, tooltip=''):
+        update_status_actions(self, ok, tooltip)
+# }}}
+
+
 class ItemsCompleter(QCompleter):  # {{{
 
     '''
@@ -671,8 +730,8 @@ class HistoryLineEdit(QComboBox):  # {{{
 
     lost_focus = pyqtSignal()
 
-    def __init__(self, *args):
-        QComboBox.__init__(self, *args)
+    def __init__(self, parent=None):
+        QComboBox.__init__(self, parent)
         self.setEditable(True)
         self.setInsertPolicy(QComboBox.InsertPolicy.NoInsert)
         self.setMaxCount(10)
@@ -682,6 +741,16 @@ class HistoryLineEdit(QComboBox):  # {{{
     def setPlaceholderText(self, txt):
         return self.lineEdit().setPlaceholderText(txt)
 
+    def contextMenuEvent(self, event):
+        menu = self.lineEdit().createStandardContextMenu()
+        menu.addSeparator()
+        menu.addAction(_('Clear history'), self.clear_history_default_impl)
+        menu.exec(event.globalPos())
+
+    def clear_history_default_impl(self):
+        self.clear()
+        history.set(self.store_name, [])
+
     @property
     def store_name(self):
         return 'lineedit_history_'+self._name
@@ -740,7 +809,7 @@ class ComboBoxWithHelp(QComboBox):  # {{{
 
     def __init__(self, parent=None):
         QComboBox.__init__(self, parent)
-        self.currentIndexChanged[int].connect(self.index_changed)
+        self.currentIndexChanged.connect(self.index_changed)
         self.help_text = ''
         self.state_set = False
 
@@ -810,7 +879,7 @@ class EncodingComboBox(QComboBox):  # {{{
 
 class PythonHighlighter(QSyntaxHighlighter):  # {{{
 
-    Rules = []
+    Rules = ()
     Formats = {}
 
     KEYWORDS = ["and", "as", "assert", "break", "class", "continue", "def",
@@ -834,39 +903,46 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
 
     def __init__(self, parent=None):
         super().__init__(parent)
+        if not self.Rules:
+            self.initialize_class_members()
 
-        self.initializeFormats()
+    @classmethod
+    def initialize_class_members(cls):
+        cls.initializeFormats()
+        r = []
 
-        PythonHighlighter.Rules.append((QRegExp(
-                "|".join([r"\b%s\b" % keyword for keyword in self.KEYWORDS])),
-                "keyword"))
-        PythonHighlighter.Rules.append((QRegExp(
-                "|".join([r"\b%s\b" % builtin for builtin in self.BUILTINS])),
-                "builtin"))
-        PythonHighlighter.Rules.append((QRegExp(
+        def a(a, b):
+            r.append((a, b))
+
+        a(re.compile(
+                "|".join([r"\b%s\b" % keyword for keyword in cls.KEYWORDS])),
+                "keyword")
+        a(re.compile(
+                "|".join([r"\b%s\b" % builtin for builtin in cls.BUILTINS])),
+                "builtin")
+        a(re.compile(
                 "|".join([r"\b%s\b" % constant
-                for constant in self.CONSTANTS])), "constant"))
-        PythonHighlighter.Rules.append((QRegExp(
+                for constant in cls.CONSTANTS])), "constant")
+        a(re.compile(
                 r"\b[+-]?[0-9]+[lL]?\b"
                 r"|\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b"
                 r"|\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b"),
-                "number"))
-        PythonHighlighter.Rules.append((QRegExp(
-                r"\bPyQt5\b|\bQt?[A-Z][a-z]\w+\b"), "pyqt"))
-        PythonHighlighter.Rules.append((QRegExp(r"\b@\w+\b"), "decorator"))
-        stringRe = QRegExp(r"""(?:'[^']*'|"[^"]*")""")
-        stringRe.setMinimal(True)
-        PythonHighlighter.Rules.append((stringRe, "string"))
-        self.stringRe = QRegExp(r"""(:?"["]".*"["]"|'''.*''')""")
-        self.stringRe.setMinimal(True)
-        PythonHighlighter.Rules.append((self.stringRe, "string"))
-        self.tripleSingleRe = QRegExp(r"""'''(?!")""")
-        self.tripleDoubleRe = QRegExp(r'''"""(?!')''')
+                "number")
+        a(re.compile(
+                r"\bPyQt6\b|\bQt?[A-Z][a-z]\w+\b"), "pyqt")
+        a(re.compile(r"\b@\w+\b"), "decorator")
+        stringRe = re.compile(r"""(?:'[^']*?'|"[^"]*?")""")
+        a(stringRe, "string")
+        cls.stringRe = re.compile(r"""(:?"["]".*?"["]"|'''.*?''')""")
+        a(cls.stringRe, "string")
+        cls.tripleSingleRe = re.compile(r"""'''(?!")""")
+        cls.tripleDoubleRe = re.compile(r'''"""(?!')''')
+        cls.Rules = tuple(r)
 
     @classmethod
     def initializeFormats(cls):
         baseFormat = QTextCharFormat()
-        baseFormat.setFontFamily('monospace')
+        baseFormat.setFontFamilies(['monospace'])
         p = QApplication.instance().palette()
         for name, color, bold, italic in (
                 ("normal", None, False, False),
@@ -896,36 +972,31 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
         prevState = self.previousBlockState()
 
         self.setFormat(0, textLength,
-                       PythonHighlighter.Formats["normal"])
+                       self.Formats["normal"])
 
         if text.startswith("Traceback") or text.startswith("Error: "):
             self.setCurrentBlockState(ERROR)
             self.setFormat(0, textLength,
-                           PythonHighlighter.Formats["error"])
+                           self.Formats["error"])
             return
         if prevState == ERROR and \
            not (text.startswith('>>>') or text.startswith("#")):
             self.setCurrentBlockState(ERROR)
             self.setFormat(0, textLength,
-                           PythonHighlighter.Formats["error"])
+                           self.Formats["error"])
             return
 
-        for regex, format in PythonHighlighter.Rules:
-            i = regex.indexIn(text)
-            while i >= 0:
-                length = regex.matchedLength()
-                self.setFormat(i, length,
-                               PythonHighlighter.Formats[format])
-                i = regex.indexIn(text, i + length)
+        for regex, fmt in PythonHighlighter.Rules:
+            for m in regex.finditer(text):
+                self.setFormat(m.start(), m.end() - m.start(), self.Formats[fmt])
 
         # Slow but good quality highlighting for comments. For more
         # speed, comment this out and add the following to __init__:
-        # PythonHighlighter.Rules.append((QRegExp(r"#.*"), "comment"))
+        # PythonHighlighter.Rules.append((re.compile(r"#.*"), "comment"))
         if not text:
             pass
         elif text[0] == "#":
-            self.setFormat(0, len(text),
-                           PythonHighlighter.Formats["comment"])
+            self.setFormat(0, len(text), self.Formats["comment"])
         else:
             stack = []
             for i, c in enumerate(text):
@@ -935,33 +1006,33 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
                     else:
                         stack.append(c)
                 elif c == "#" and len(stack) == 0:
-                    self.setFormat(i, len(text),
-                                   PythonHighlighter.Formats["comment"])
+                    self.setFormat(i, len(text), self.Formats["comment"])
                     break
 
         self.setCurrentBlockState(NORMAL)
 
-        if self.stringRe.indexIn(text) != -1:
+        if self.stringRe.search(text) is not None:
             return
         # This is fooled by triple quotes inside single quoted strings
-        for i, state in ((self.tripleSingleRe.indexIn(text),
-                          TRIPLESINGLE),
-                         (self.tripleDoubleRe.indexIn(text),
-                          TRIPLEDOUBLE)):
+        for m, state in (
+            (self.tripleSingleRe.search(text), TRIPLESINGLE),
+            (self.tripleDoubleRe.search(text), TRIPLEDOUBLE)
+        ):
+            i = -1 if m is None else m.start()
             if self.previousBlockState() == state:
                 if i == -1:
                     i = len(text)
                     self.setCurrentBlockState(state)
                 self.setFormat(0, i + 3,
-                               PythonHighlighter.Formats["string"])
+                               self.Formats["string"])
             elif i > -1:
                 self.setCurrentBlockState(state)
                 self.setFormat(i, len(text),
-                               PythonHighlighter.Formats["string"])
+                               self.Formats["string"])
 
     def rehighlight(self):
         QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
-        QSyntaxHighlighter.rehighlight(self)
+        super().rehighlight()
         QApplication.restoreOverrideCursor()
 
 # }}}
@@ -969,13 +1040,12 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
 
 # Splitter {{{
 
-
 class SplitterHandle(QSplitterHandle):
 
     double_clicked = pyqtSignal(object)
 
     def __init__(self, orientation, splitter):
-        QSplitterHandle.__init__(self, orientation, splitter)
+        super().__init__(orientation, splitter)
         splitter.splitterMoved.connect(self.splitter_moved,
                 type=Qt.ConnectionType.QueuedConnection)
         self.double_clicked.connect(splitter.double_clicked,
@@ -998,7 +1068,7 @@ class LayoutButton(QToolButton):
     def __init__(self, icon, text, splitter=None, parent=None, shortcut=None):
         QToolButton.__init__(self, parent)
         self.label = text
-        self.setIcon(QIcon(icon))
+        self.setIcon(QIcon.ic(icon))
         self.setCheckable(True)
         self.icname = os.path.basename(icon).rpartition('.')[0]
 
@@ -1020,7 +1090,7 @@ class LayoutButton(QToolButton):
         t = _('Hide {}') if self.isChecked() else _('Show {}')
         t = t.format(self.label)
         if self.shortcut:
-            t += ' [{}]'.format(self.shortcut)
+            t += f' [{self.shortcut}]'
         self.setText(t), self.setToolTip(t), self.setStatusTip(t)
 
     def set_state_to_show(self, *args):
@@ -1064,7 +1134,7 @@ class Splitter(QSplitter):
             initial_side_size=120, connect_button=True,
             orientation=Qt.Orientation.Horizontal, side_index=0, parent=None,
             shortcut=None, hide_handle_on_single_panel=True):
-        QSplitter.__init__(self, parent)
+        super().__init__(parent)
         self.reapply_sizes.connect(self.setSizes, type=Qt.ConnectionType.QueuedConnection)
         self.hide_handle_on_single_panel = hide_handle_on_single_panel
         if hide_handle_on_single_panel:
@@ -1088,7 +1158,7 @@ class Splitter(QSplitter):
             self.button.clicked.connect(self.double_clicked)
 
         if shortcut is not None:
-            self.action_toggle = QAction(QIcon(icon), _('Toggle') + ' ' + label,
+            self.action_toggle = QAction(QIcon.ic(icon), _('Toggle') + ' ' + label,
                     self)
             self.action_toggle.changed.connect(self.update_shortcut)
             self.action_toggle.triggered.connect(self.toggle_triggered)
@@ -1176,18 +1246,27 @@ class Splitter(QSplitter):
             # when the event loop ticks
             self.reapply_sizes.emit(sizes)
 
+    def ignore_child_paints(self, ignore=True):
+        for widget in self:
+            if hasattr(widget, 'ignore_paint_events'):
+                widget.ignore_paint_events = ignore
+
     def do_resize(self, *args):
         orig = self.desired_side_size
-        QSplitter.resizeEvent(self, self._resize_ev)
+        super().resizeEvent(self._resize_ev)
         if orig > 20 and self.desired_show:
             c = 0
             while abs(self.side_index_size - orig) > 10 and c < 5:
                 self.apply_state(self.get_state(), save_desired=False)
                 c += 1
+        self.ignore_child_paints(False)
+
+    def __iter__(self):
+        for i in range(self.count()):
+            yield self.widget(i)
 
     def resizeEvent(self, ev):
-        if self.resize_timer.isActive():
-            self.resize_timer.stop()
+        self.ignore_child_paints()
         self._resize_ev = ev
         self.resize_timer.start()
 
@@ -1266,15 +1345,15 @@ class PaperSizes(QComboBox):  # {{{
             if iswindows or ismacos:
                 # On Linux, this can cause Qt to load the system cups plugin
                 # which can crash: https://bugs.launchpad.net/calibre/+bug/1861741
-                PaperSizes.system_default_paper_size = 'letter' if QPrinter().pageSize() == QPagedPaintDevice.PageSize.Letter else 'a4'
+                PaperSizes.system_default_paper_size = 'letter' if QPrinter().pageLayout().pageSize().id() == QPageSize.PageSizeId.Letter else 'a4'
         if not choices:
             from calibre.ebooks.conversion.plugins.pdf_output import PAPER_SIZES
             choices = PAPER_SIZES
         for a in sorted(choices, key=numeric_sort_key):
-            s = getattr(QPageSize, a.capitalize())
+            s = getattr(QPageSize.PageSizeId, a.capitalize())
             sz = QPageSize.definitionSize(s)
             unit = {QPageSize.Unit.Millimeter: 'mm', QPageSize.Unit.Inch: 'inch'}[QPageSize.definitionUnits(s)]
-            name = '{} ({:g} x {:g} {})'.format(QPageSize.name(s), sz.width(), sz.height(), unit)
+            name = f'{QPageSize.name(s)} ({sz.width():g} x {sz.height():g} {unit})'
             self.addItem(name, a)
 
     @property
@@ -1290,6 +1369,16 @@ class PaperSizes(QComboBox):  # {{{
 # }}}
 
 
+class BusyCursor:  # {{{
+
+    def __enter__(self):
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
+
+    def __exit__(self, *args):
+        QApplication.restoreOverrideCursor()
+# }}}
+
+
 if __name__ == '__main__':
     from qt.core import QTextEdit
     app = QApplication([])
@@ -1298,4 +1387,4 @@ if __name__ == '__main__':
     # w.setSyntaxHighlighter(s)
     w.setText(open(__file__, 'rb').read().decode('utf-8'))
     w.show()
-    app.exec_()
+    app.exec()
